#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The jsonpath interface module."""
from jsonpath2.path import Path


def parse(jsonpath_str):
    """Parse the json path."""
    return Path.parse_str(jsonpath_str)


def find(expr, data):
    """Match the expression in the data and return truthy value."""
    return bool(list(expr.match(data)))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test script to run the command interface for testing."""
from __future__ import print_function
import sys
import os
from unittest import TestCase
from tempfile import mkdtemp
from shutil import rmtree
try:
    import sh
except ImportError:
    import pbs

    class Sh:
        """Sh style wrapper."""

        def __getattr__(self, attr):
            """Return command object like sh."""
            return pbs.Command(attr)

        # pylint: disable=invalid-name
        @staticmethod
        def Command(attr):
            """Return command object like sh."""
            return pbs.Command(attr)
    sh = Sh()
import peewee
from pacifica.notifications.orm import EventMatch, NotificationSystem, EventLogMatch, EventLog
from pacifica.notifications.__main__ import cmd, main


class TestAdminCmdBase:
    """Test base class to setup update conditions."""

    virtualenv_dir = mkdtemp()

    @classmethod
    def _setup_venv(cls):
        """Setup the Python VirtualEnv."""
        sh.Command(sys.executable)('-m', 'virtualenv', '--python', sys.executable, cls.virtualenv_dir)

    @classmethod
    def _get_python_cmd(cls):
        """Get the Python Command object."""
        python_exe = os.path.basename(sys.executable)
        python_venv_cmd = None
        for exe_dir in ['bin', 'Scripts']:
            fpath = os.path.join(cls.virtualenv_dir, exe_dir, python_exe)
            if os.path.isfile(fpath) and os.access(fpath, os.X_OK):
                python_venv_cmd = sh.Command(fpath)
        return python_venv_cmd

    # pylint: disable=invalid-name
    @classmethod
    def setUpClass(cls):
        """Setup a virtualenv and install the original version."""
        cls._setup_venv()
        cls._get_python_cmd()('-m', 'pip', 'install', 'pacifica-notifications==0.2.3', 'psycopg2')

    @classmethod
    def setUp(cls):
        """Setup the db for the test."""
        EventLogMatch.drop_table(safe=True)
        EventMatch.drop_table(safe=True)
        EventLog.drop_table(safe=True)
        NotificationSystem.drop_table(safe=True)
        cls._get_python_cmd()(
            '-c', 'import sys; from pacifica.notifications.orm import database_setup; sys.exit(database_setup())'
        )

    @classmethod
    def tearDownClass(cls):
        """Remove the virtualenv dir."""
        rmtree(cls.virtualenv_dir)


class TestAdminCmdSync(TestAdminCmdBase, TestCase):
    """Test the database upgrade scripting."""

    def test_dbchk(self):
        """Test that dbchk doesn't work."""
        self.assertEqual(cmd('dbchk'), -1)

    def test_dbchk_equal(self):
        """Test that dbchk doesn't work."""
        self.assertEqual(cmd('dbchk', '--equal'), -1)

    def test_main_error(self):
        """Test that dbchk doesn't work."""
        with self.assertRaises(peewee.OperationalError):
            main('--stop-after-a-moment')

    def test_dbsync(self):
        """Run the update by calling dbsync."""
        self.assertEqual(cmd('dbsync'), 0)

    def test_main(self):
        """Test that dbchk doesn't work."""
        cmd('dbsync')
        cmd('dbsync')
        hit_exception = False
        try:
            main('--stop-after-a-moment', '--cpconfig', os.path.join(os.path.dirname(__file__), '..', 'server.conf'))
        # pylint: disable=broad-except
        except Exception:
            hit_exception = True
        self.assertFalse(hit_exception)


class TestAdminCmdEvents(TestAdminCmdBase, TestCase):
    """Test the database upgrade scripting."""

    event_match_objs = [
        {
            'uuid': '7e487ebb-309f-4dd7-bb17-0287b4209e2e',
            'name': 'Event Match 01',
            'jsonpath': '$["data"]',
            'user': 'johndoe',
            'target_url': 'http://localhost/events'
        },
        {
            'uuid': '45736222-a4d5-48dd-99e2-ea54ae7f6bb7',
            'name': 'Event Match 02',
            'jsonpath': '$["data"]',
            'user': 'janedoe',
            'target_url': 'http://localhost/events'
        }
    ]
    event_log_objs = [
        {
            'uuid': '9419b5b6-6f11-439a-9585-42e272c97da4',
            'jsondata': '{}'
        },
        {
            'uuid': '474549ce-672c-4771-8b39-c4ecdd951691',
            'jsondata': '{}'
        },
        {
            'uuid': '390d52b7-0c50-415c-b750-2071520f05ed',
            'jsondata': '{}'
        }
    ]
    event_log_match_objs = [
        {
            'uuid': '40cab09f-547a-432f-8424-bd75a6993170',
            'event_log': '390d52b7-0c50-415c-b750-2071520f05ed',
            'event_match': '7e487ebb-309f-4dd7-bb17-0287b4209e2e',
            'policy_status_code': '201',
            'policy_resp_body': '{"something": "policy returned"}'
        },
        {
            'uuid': '4728abdb-58e7-4ee2-81a8-ce4ae69eeda4',
            'event_log': '474549ce-672c-4771-8b39-c4ecdd951691',
            'event_match': '45736222-a4d5-48dd-99e2-ea54ae7f6bb7',
            'policy_status_code': '201',
            'policy_resp_body': '{"something": "policy returned"}'
        },
        {
            'uuid': '0ebfebdd-7de7-47fd-bace-5294f13ef5c0',
            'event_log': '9419b5b6-6f11-439a-9585-42e272c97da4',
            'event_match': '45736222-a4d5-48dd-99e2-ea54ae7f6bb7',
            'policy_status_code': '201',
            'policy_resp_body': '{"something": "policy returned"}'

        }
    ]

    @classmethod
    def setUp(cls):
        """Call super method and create some data in the db."""
        super().setUp()
        cmd('dbsync')
        for event_match_hash in cls.event_match_objs:
            event_match_obj = EventMatch(**event_match_hash)
            event_match_obj.save(force_insert=True)
        for event_log_hash in cls.event_log_objs:
            event_log_obj = EventLog(**event_log_hash)
            event_log_obj.save(force_insert=True)
        for event_log_match_hash in cls.event_log_match_objs:
            event_log_match_obj = EventLogMatch(**event_log_match_hash)
            event_log_match_obj.save(force_insert=True)

    def test_eventget(self):
        """Test that dbchk doesn't work."""
        self.assertEqual(cmd('eventget'), 0)

    def test_eventget_with_args(self):
        """Test some of the arguments with eventget."""
        self.assertEqual(cmd('eventget', '--start-date', '2020-01-01 00:00:00', '--end-date', '2020-02-01 00:00:00'), 0)

    def test_eventget_wrong_order(self):
        """Test the reverse of start and end time."""
        with self.assertRaises(ValueError):
            cmd('eventget', '--start-date', '2020-03-01 00:00:00', '--end-date', '2020-02-01 00:00:00')

    def test_eventget_with_uuids(self):
        """Test some of the uuid arguments with eventget."""
        self.assertEqual(cmd('eventget', '9419b5b6-6f11-439a-9585-42e272c97da4'), 0)

    def test_eventpurge_with_uuids(self):
        """Test some of the uuid arguments with eventpurge."""
        self.assertEqual(cmd('eventpurge', '9419b5b6-6f11-439a-9585-42e272c97da4'), 0)

    def test_eventpurge_with_args(self):
        """Test some of the uuid arguments with eventpurge."""
        self.assertEqual(cmd('eventpurge', '--older-than-date', '2199-01-01 00:00:00'), 0)

    def test_eventpurge(self):
        """Test some eventpurge."""
        self.assertEqual(cmd('eventpurge'), 0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy module containing classes for rest interface."""
from uuid import UUID
from datetime import datetime
from json import dumps, loads
from jsonschema import validate
import cherrypy
from cherrypy import HTTPError
from peewee import DoesNotExist
from pacifica.notifications import orm
from pacifica.notifications.config import get_config
from pacifica.notifications.tasks import dispatch_event


def encode_text(thing_obj):
    """Encode the text to bytes."""
    return bytes(thing_obj, 'utf8')  # pragma: no cover only for python 3


def get_remote_user():
    """Get the remote user from cherrypy request headers."""
    return cherrypy.request.headers.get(
        get_config().get('notifications', 'user_header'),
        get_config().get('DEFAULT', 'default_user')
    )


def error_page_default(**kwargs):
    """The default error page should always enforce json."""
    cherrypy.response.headers['Content-Type'] = 'application/json'
    return dumps({
        'status': kwargs['status'],
        'message': kwargs['message'],
        'traceback': kwargs['traceback'],
        'version': kwargs['version']
    })


class EventMatch:
    """CherryPy EventMatch endpoint."""

    exposed = True
    json_schema = {
        'definitions': {
            'eventmatch': {
                'type': 'object',
                'properties': {
                    'uuid': {'type': 'string'},
                    'name': {'type': 'string'},
                    'jsonpath': {'type': 'string'},
                    'user': {'type': 'string'},
                    'disabled': {'type': ['string', 'null'], 'format': 'date-time'},
                    'error': {'type': ['string', 'null']},
                    'target_url': {'type': 'string'},
                    'version': {'type': 'string'},
                    'extensions': {'type': 'object'},
                    'auth': {
                        'type': 'object',
                        'properties': {
                            'type': {
                                'type': 'string',
                                'enum': ['basic', 'header']
                            },
                            'basic': {
                                'type': 'object',
                                'properties': {
                                    'username': {'type': 'string'},
                                    'password': {'type': 'string'}
                                },
                                'required': ['username', 'password']
                            },
                            'header': {
                                'type': 'object',
                                'properties': {
                                    'type': {'type': 'string'},
                                    'credentials': {'type': 'string'}
                                },
                                'required': ['type', 'credentials']
                            },
                        },
                        'required': ['type']
                    },
                    'created': {'type': 'string', 'format': 'date-time'},
                    'updated': {'type': 'string', 'format': 'date-time'},
                    'deleted': {'type': ['string', 'null'], 'format': 'date-time'}
                }
            }
        },
        '$ref': '#/definitions/eventmatch',
        'not': {
            'required': ['uuid', 'user', 'created', 'updated', 'deleted', 'version']
        }
    }

    @staticmethod
    def _http_get(event_uuid):
        """Internal get event by UUID and return peewee obj."""
        cherrypy.response.headers['Content-Type'] = 'application/json'
        orm.EventMatch.database_connect()
        try:
            event_obj = orm.EventMatch.get(
                orm.EventMatch.uuid == UUID('{{{}}}'.format(event_uuid)))
        except DoesNotExist:
            orm.EventMatch.database_close()
            raise HTTPError(403, 'Forbidden')
        orm.EventMatch.database_close()
        if event_obj.user != get_remote_user() or event_obj.deleted:
            raise HTTPError(403, 'Forbidden')
        return event_obj

    @classmethod
    # pylint: disable=invalid-name
    def GET(cls, event_uuid=None):
        """Get the event ID and return it."""
        if event_uuid:
            objs = cls._http_get(event_uuid).to_hash()
        else:
            cherrypy.response.headers['Content-Type'] = 'application/json'
            orm.EventMatch.database_connect()
            query = orm.EventMatch.select().where(
                (orm.EventMatch.user == get_remote_user()) &
                (orm.EventMatch.deleted >> None)
            )
            objs = [x.to_hash() for x in query]
            orm.EventMatch.database_close()
        if objs:
            return encode_text(dumps(objs))
        raise HTTPError(403, 'Forbidden')

    @classmethod
    # pylint: disable=invalid-name
    def PUT(cls, event_uuid):
        """Update an Event Match obj in the database."""
        event_obj = cls._http_get(event_uuid)
        json_obj = loads(cherrypy.request.body.read().decode('utf8'))
        validate(json_obj, cls.json_schema)
        json_obj['extensions'] = dumps(json_obj.get('extensions', {}))
        json_obj['auth'] = dumps(json_obj.get('auth', {}))
        for key, value in json_obj.items():
            setattr(event_obj, key, value)
        event_obj.updated = datetime.now()
        event_obj.validate_jsonpath()
        orm.EventMatch.database_connect()
        with orm.EventMatch.atomic():
            event_obj.save()
        orm.EventMatch.database_close()
        return cls.GET(str(event_obj.uuid))

    @classmethod
    # pylint: disable=invalid-name
    def POST(cls):
        """Create an Event Match obj in the database."""
        orm.EventMatch.database_connect()
        event_match_obj = loads(cherrypy.request.body.read().decode('utf8'))
        validate(event_match_obj, cls.json_schema)
        event_match_obj['extensions'] = dumps(
            event_match_obj.get('extensions', {})
        )
        event_match_obj['auth'] = dumps(event_match_obj.get('auth', {}))
        event_match_obj['user'] = get_remote_user()
        event_obj = orm.EventMatch(**event_match_obj)
        event_obj.validate_jsonpath()
        with orm.EventMatch.atomic():
            event_obj.save(force_insert=True)
        orm.EventMatch.database_close()
        return cls.GET(str(event_obj.uuid))

    @classmethod
    # pylint: disable=invalid-name
    def DELETE(cls, event_uuid):
        """Delete the event by uuid."""
        event_obj = cls._http_get(event_uuid)
        orm.EventMatch.database_connect()
        event_obj.deleted = datetime.now()
        event_obj.updated = datetime.now()
        with orm.EventMatch.atomic():
            event_obj.save()
        orm.EventMatch.database_close()


# pylint: disable=too-few-public-methods
class ReceiveEvent:
    """CherryPy Receive Event object."""

    exposed = True
    event_json_schema = {}

    @classmethod
    # pylint: disable=invalid-name
    def POST(cls):
        """Receive the event and dispatch it to backend."""
        event_obj = loads(cherrypy.request.body.read().decode('utf8'))
        validate(event_obj, cls.event_json_schema)
        return encode_text(str(dispatch_event.delay(event_obj)))
# pylint: enable=too-few-public-methods


# pylint: disable=too-few-public-methods
class Root:
    """CherryPy Root Object."""

    exposed = True
    eventmatch = EventMatch()
    receive = ReceiveEvent()
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The ORM module defining the SQL model for notifications."""
import uuid
from time import sleep
from datetime import datetime
from json import loads
from peewee import Model, CharField, TextField, DateTimeField, UUIDField
from peewee import OperationalError, IntegerField, ForeignKeyField
from playhouse.migrate import SchemaMigrator, migrate
from playhouse.db_url import connect
from .config import get_config
from .jsonpath import parse

SCHEMA_MAJOR = 3
SCHEMA_MINOR = 0
DB = connect(get_config().get('database', 'peewee_url'))


class OrmSync:
    """
    Special module for syncing the orm.

    This module should incorporate a schema migration strategy.

    The supported versions migrating forward must be in a versions array
    containing tuples for major and minor versions.

    The version tuples are directly translated to method names in the
    orm_update class for the update between those versions.

    Example Version Control::

      class orm_update:
        versions = [
          (0, 1),
          (0, 2),
          (1, 0),
          (1, 1)
        ]

        def update_0_1_to_0_2():
            pass
        def update_0_2_to_1_0():
            pass

    The body of the update should follow peewee migration practices.
    http://docs.peewee-orm.com/en/latest/peewee/playhouse.html#migrate
    """

    versions = [
        (0, 0),
        (1, 0),
        (2, 0),
        (3, 0)
    ]

    @staticmethod
    def dbconn_blocking():
        """Wait for the db connection."""
        dbcon_attempts = get_config().getint('database', 'connect_attempts')
        dbcon_wait = get_config().getint('database', 'connect_wait')
        while dbcon_attempts:
            try:
                EventMatch.database_connect()
                return
            except OperationalError:
                # couldnt connect, potentially wait and try again
                sleep(dbcon_wait)
                dbcon_attempts -= 1
        raise OperationalError('Failed database connect retry.')

    @classmethod
    def update_0_0_to_1_0(cls):
        """Update by adding the new table."""
        if not EventMatch.table_exists():
            EventMatch.create_table()
            migrator = SchemaMigrator(DB)
            migrate(migrator.drop_column('eventmatch', 'auth'))

    @classmethod
    def update_1_0_to_2_0(cls):
        """Update by adding the auth column."""
        migrator = SchemaMigrator(DB)
        migrate(
            migrator.add_column(
                'eventmatch',
                'auth',
                TextField(null=True)
            )
        )

    @classmethod
    def update_2_0_to_3_0(cls):
        """Update by adding the auth column."""
        for orm_obj in [EventLog, EventLogMatch]:
            if not orm_obj.table_exists():
                orm_obj.create_table()

    @classmethod
    def update_tables(cls):
        """Update the database to the current version."""
        verlist = cls.versions
        db_ver = NotificationSystem.get_version()
        if verlist.index(verlist[-1]) == verlist.index(db_ver):
            # we have the current version don't update
            return True
        with EventMatch.atomic():
            for db_ver in verlist[verlist.index(db_ver):-1]:
                next_db_ver = verlist[verlist.index(db_ver)+1]
                method_name = 'update_{}_to_{}'.format(
                    '{}_{}'.format(*db_ver),
                    '{}_{}'.format(*next_db_ver)
                )
                getattr(cls, method_name)()
            NotificationSystem.drop_table()
            NotificationSystem.create_table()
            NotificationSystem.get_or_create_version()
        return True


# pylint: disable=too-few-public-methods
class BaseModel(Model):
    """Auto-generated by pwiz."""

    class Meta:
        """Map to the database connected above."""

        database = DB

    @classmethod
    def database_connect(cls):
        """
        Make sure database is connected.

        Trying to connect a second
        time *does* cause problems.
        """
        # pylint: disable=no-member
        if not cls._meta.database.is_closed():
            cls._meta.database.close()
        cls._meta.database.connect()
        # pylint: enable=no-member

    @classmethod
    def database_close(cls):
        """
        Close the database connection.

        Closing already closed database
        is not a problem, so continue on.
        """
        # pylint: disable=no-member
        if not cls._meta.database.is_closed():
            cls._meta.database.close()
        # pylint: enable=no-member

    @classmethod
    def atomic(cls):
        """Do the database atomic action."""
        return cls._meta.database.atomic()


class NotificationSystem(BaseModel):
    """Notification Schema Version Model."""

    part = CharField(primary_key=True)
    value = IntegerField(default=-1)

    @classmethod
    def get_or_create_version(cls):
        """Set or create the current version of the schema."""
        if not cls.table_exists():
            return (0, 0)
        major = cls.get_or_create(part='major', value=SCHEMA_MAJOR)
        minor = cls.get_or_create(part='minor', value=SCHEMA_MINOR)
        return (major, minor)

    @classmethod
    def get_version(cls):
        """Get the current version as a tuple."""
        if not cls.table_exists():
            return (0, 0)
        return (cls.get(part='major').value, cls.get(part='minor').value)

    @classmethod
    def is_equal(cls):
        """Check to see if schema version matches code version."""
        major, minor = cls.get_version()
        return major == SCHEMA_MAJOR and minor == SCHEMA_MINOR

    @classmethod
    def is_safe(cls):
        """Check to see if the schema version is safe for the code."""
        major, _minor = cls.get_version()
        return major == SCHEMA_MAJOR


class EventLog(BaseModel):
    """Events matching via jsonpath per user."""

    uuid = UUIDField(primary_key=True, default=uuid.uuid4, index=True)
    jsondata = TextField()
    created = DateTimeField(default=datetime.now, index=True)

    # pylint: disable=too-few-public-methods
    class Meta:
        """The meta class that contains db connection."""

        database = DB
    # pylint: enable=too-few-public-methods


class EventMatch(BaseModel):
    """Events matching via jsonpath per user."""

    uuid = UUIDField(primary_key=True, default=uuid.uuid4, index=True)
    name = CharField(index=True)
    jsonpath = TextField()
    user = CharField(index=True)
    disabled = DateTimeField(index=True, null=True, default=None)
    error = TextField(null=True)
    target_url = TextField()
    version = CharField(default='v0.1')
    # JSONField is extension specific and we are using URLs to bind
    # to a database backend
    extensions = TextField(default='{}')
    auth = TextField(default='{}', null=True)
    created = DateTimeField(default=datetime.now, index=True)
    updated = DateTimeField(default=datetime.now, index=True)
    deleted = DateTimeField(null=True, index=True)

    # pylint: disable=too-few-public-methods
    class Meta:
        """The meta class that contains db connection."""

        database = DB
    # pylint: enable=too-few-public-methods

    def validate_jsonpath(self):
        """Validate the jsonpath string."""
        parse(self.jsonpath)
        return True

    def to_hash(self):
        """Convert the object to a json serializable hash."""
        ret_obj = {}
        for field_name in self._meta.sorted_field_names:
            ret_obj[field_name] = getattr(self, field_name)
        ret_obj['uuid'] = str(ret_obj['uuid'])
        ret_obj['extensions'] = loads(ret_obj['extensions'])
        ret_obj['auth'] = loads(ret_obj['auth'])
        for dt_element in ['disabled', 'deleted', 'updated', 'created']:
            if getattr(self, dt_element):
                # pylint: disable=no-member
                ret_obj[dt_element] = getattr(self, dt_element).isoformat()
                # pylint: enable=no-member
        return ret_obj


class EventLogMatch(BaseModel):
    """Events matching via jsonpath per user."""

    uuid = UUIDField(primary_key=True, default=uuid.uuid4, index=True)
    event_log = ForeignKeyField(EventLog, backref='event_matches')
    event_match = ForeignKeyField(EventMatch, backref='event_logs')
    policy_status_code = TextField()
    policy_resp_body = TextField()
    target_status_code = TextField(default='')
    target_resp_body = TextField(default='')
    created = DateTimeField(default=datetime.now, index=True)

    # pylint: disable=too-few-public-methods
    class Meta:
        """The meta class that contains db connection."""

        database = DB
    # pylint: enable=too-few-public-methods


def eventget(args):
    """Get events based on command line argument in args."""
    query = EventLog.select()
    if args.events:
        query = query.where(EventLog.uuid << args.events)
    else:
        query = query.where(
            (EventLog.created < args.end) &
            (EventLog.created > args.start)
        ).limit(args.limit)
    for event_obj in query:
        print('Event - {}\n{}'.format(event_obj.uuid, event_obj.jsondata))
        for elm_obj in event_obj.event_matches:
            print('    ELM {} ({}) policy {} target {}'.format(
                elm_obj.event_match.uuid,
                elm_obj.created.isoformat(),
                elm_obj.policy_status_code,
                elm_obj.target_status_code
            ))
    return True


def eventpurge(args):
    """Purge events based on command line argument in args."""
    query = EventLog.select()
    if args.events:
        query = query.where(EventLog.uuid << args.events)
    else:
        query = query.where(
            (EventLog.created < args.older_than)
        )
    for event_obj in query:
        for elm_obj in event_obj.event_matches:
            elm_obj.delete_instance()
        event_obj.delete_instance()
    return True



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The Celery tasks module."""
from datetime import datetime
from json import dumps, loads
import requests
from requests.exceptions import RequestException
from celery import Celery
from .orm import EventMatch, EventLog, EventLogMatch
from .jsonpath import parse, find
from .config import get_config

CELERY_APP = Celery(
    'notifications',
    broker=get_config().get('celery', 'broker_url'),
    backend=get_config().get('celery', 'backend_url')
)


@CELERY_APP.task
def dispatch_event(event_obj):
    """Get all the events and see which match."""
    EventLog.database_connect()
    orm_event = EventLog.create(
        jsondata=dumps(event_obj)
    )
    orm_event.save()
    EventLog.database_close()
    dispatch_orm_event(orm_event)


def dispatch_orm_event(orm_event):
    """Dispatch the event from an existing orm obj."""
    results = []
    event_obj = loads(orm_event.jsondata)
    EventMatch.database_connect()
    eventmatch_objs = EventMatch.select().where(
        (EventMatch.deleted >> None) &
        (EventMatch.disabled >> None)
    )
    EventMatch.database_close()
    for eventmatch in eventmatch_objs:
        jsonpath_expr = parse(eventmatch.jsonpath)
        if find(jsonpath_expr, event_obj):
            results.append(query_policy.delay(eventmatch.to_hash(), event_obj, orm_event.uuid))
    return results


def disable_eventmatch(eventmatch_uuid, error):
    """Disable the eventmatch obj."""
    EventMatch.database_connect()
    with EventMatch.atomic():
        eventmatch = EventMatch.get(EventMatch.uuid == eventmatch_uuid)
        eventmatch.disabled = datetime.now()
        eventmatch.error = error
        eventmatch.save()
    EventMatch.database_close()


def create_log_match(eventmatch, event_log_uuid, policy_resp):
    """Create the EventLogMatch object."""
    EventLogMatch.database_connect()
    orm_elm = EventLogMatch.create(
        event_log=event_log_uuid,
        event_match=eventmatch['uuid'],
        policy_status_code=policy_resp.status_code,
        policy_resp_body=policy_resp.text
    )
    orm_elm.save()
    EventLogMatch.database_close()
    return orm_elm.uuid


def update_log_match(elm_uuid, target_resp):
    """Update the EventLogMatch object with the target resp."""
    EventLogMatch.database_connect()
    orm_elm = EventLogMatch.get_by_id(elm_uuid)
    orm_elm.target_status_code = target_resp.status_code
    orm_elm.target_resp_body = target_resp.text
    orm_elm.save()
    EventLogMatch.database_close()


@CELERY_APP.task
def query_policy(eventmatch, event_obj, event_log_uuid):
    """Query policy server to see if the event should be routed."""
    resp = requests.post(
        '{}/events/{}'.format(
            get_config().get('notifications', 'policy_url'),
            eventmatch['user']
        ),
        data=dumps(event_obj),
        headers={'Content-Type': 'application/json'}
    )
    resp_major = int(int(resp.status_code)/100)
    if resp_major == 5:
        create_log_match(eventmatch, event_log_uuid, resp)
        disable_eventmatch(eventmatch['uuid'], resp.text)
    if resp_major == 4:
        return
    if resp_major == 2:
        elm_uuid = create_log_match(eventmatch, event_log_uuid, resp)
        route_event.delay(eventmatch, event_obj, elm_uuid)


def event_auth_to_requests(eventmatch, headers):
    """Convert the eventmatch authentication to requests arguments."""
    requests_kwargs = {}
    if eventmatch.get('auth').get('type', None) == 'basic':
        auth_obj = eventmatch.get('auth').get('basic', {})
        requests_kwargs['auth'] = (auth_obj.get('username', ''), auth_obj.get('password', ''))
    elif eventmatch.get('auth').get('type', None) == 'header':
        auth_obj = eventmatch.get('auth').get('header', {})
        headers['Authorization'] = '{} {}'.format(
            auth_obj.get('type', ''),
            auth_obj.get('credentials', '')
        )
    return requests_kwargs


@CELERY_APP.task
def route_event(eventmatch, event_obj, elm_uuid):
    """Route the event to the target url."""
    try:
        new_extensions = event_obj.get('extensions', {})
        new_extensions.update(eventmatch.get('extensions', {}))
        event_obj['extensions'] = new_extensions
        headers = {'Content-Type': 'application/json'}
        extra_args = event_auth_to_requests(eventmatch, headers)
        resp = requests.post(
            eventmatch['target_url'],
            data=dumps(event_obj),
            headers=headers,
            **extra_args
        )
    except RequestException as ex:
        disable_eventmatch(eventmatch['uuid'], str(ex))
        return
    resp_major = int(int(resp.status_code)/100)
    if resp_major in (5, 4):
        disable_eventmatch(eventmatch['uuid'], resp.text)
    update_log_match(elm_uuid, resp)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the example module."""
from argparse import Namespace
from os.path import join, dirname
from datetime import datetime
from time import sleep
from json import loads, dumps
import requests
from cherrypy.test import helper
from pacifica.notifications.orm import EventMatch, EventLog
from pacifica.notifications.__main__ import _eventretry
from .common_test import NotificationsCPTest, eventmatch_droptables


class CeleryCPTest(NotificationsCPTest, helper.CPWebCase):
    """Test the EventMatch class."""

    @eventmatch_droptables
    def test_bad_url(self):
        """Test the create POST method in EventMatch."""
        uuid_list = []
        resp = self._create_eventmatch(
            headers={'Http-Remote-User': 'doesnotexist'}
        )
        eventmatch_obj = resp.json()
        self.assertEqual(eventmatch_obj['user'], 'doesnotexist')
        uuid_list.append(eventmatch_obj['uuid'])
        resp = self._create_eventmatch(
            headers={'Http-Remote-User': 'bjohn'}
        )
        eventmatch_obj = resp.json()
        self.assertEqual(eventmatch_obj['user'], 'bjohn')
        resp = self._create_eventmatch(
            headers={'Http-Remote-User': 'dmlb2001'},
            json_data={'target_url': 'http://127.0.0.1:8080/something/no/where'}
        )
        eventmatch_obj = resp.json()
        self.assertEqual(eventmatch_obj['user'], 'dmlb2001')
        self.assertEqual(
            eventmatch_obj['target_url'], 'http://127.0.0.1:8080/something/no/where'
        )
        uuid_list.append(eventmatch_obj['uuid'])
        event_obj = loads(open(join(dirname(__file__), 'test_files', 'events.json')).read())
        resp = requests.post(
            '{}/receive'.format(self.url),
            data=dumps(event_obj),
            headers={'Content-Type': 'application/cloudevents+json; charset=utf-8'}
        )
        self.assertEqual(resp.status_code, 200)
        sleep(10)
        for uuid in uuid_list:
            eventmatch_obj = EventMatch.get(EventMatch.uuid == uuid)
            self.assertEqual(eventmatch_obj.disabled.year, datetime.now().year)

    @eventmatch_droptables
    def test_bad_connection(self):
        """Test the create POST method in EventMatch."""
        resp = self._create_eventmatch(
            headers={'Http-Remote-User': 'dmlb2001'},
            json_data={
                'target_url': 'http://127.0.0.1:8192/something/no/where',
                'auth': {
                    'type': 'header',
                    'header': {
                        'type': 'Bearer',
                        'credentials': 'somerandomsharedsecret'
                    }
                }
            }
        )
        eventmatch_obj = resp.json()
        self.assertEqual(eventmatch_obj['user'], 'dmlb2001')
        self.assertEqual(
            eventmatch_obj['target_url'], 'http://127.0.0.1:8192/something/no/where'
        )
        event_obj = loads(open(join(dirname(__file__), 'test_files', 'events.json')).read())
        resp = requests.post(
            '{}/receive'.format(self.url),
            data=dumps(event_obj),
            headers={'Content-Type': 'application/cloudevents+json; charset=utf-8'}
        )
        self.assertEqual(resp.status_code, 200)
        sleep(10)
        EventMatch.database_connect()
        eventmatch_obj = EventMatch.get(
            EventMatch.uuid == eventmatch_obj['uuid']
        )
        EventMatch.database_close()
        self.assertEqual(eventmatch_obj.disabled.year, datetime.now().year)

    @eventmatch_droptables
    def test_create(self):
        """Test the create POST method in EventMatch."""
        resp = self._create_eventmatch(
            headers={'Http-Remote-User': 'dmlb2001'}
        )
        eventmatch_obj = resp.json()
        self.assertEqual(eventmatch_obj['user'], 'dmlb2001')
        event_obj = loads(open(join(dirname(__file__), 'test_files', 'events.json')).read())
        resp = requests.post(
            '{}/receive'.format(self.url),
            data=dumps(event_obj),
            headers={'Content-Type': 'application/cloudevents+json; charset=utf-8'}
        )
        self.assertEqual(resp.status_code, 200)
        sleep(10)
        EventMatch.database_connect()
        eventmatch_obj = EventMatch.get(
            EventMatch.uuid == eventmatch_obj['uuid']
        )
        EventMatch.database_close()
        self.assertEqual(eventmatch_obj.disabled, None)
        EventLog.database_connect()
        eventlog_obj = EventLog.get()
        EventLog.database_close()
        fake_args = Namespace(events=[eventlog_obj.uuid])
        self.assertEqual(_eventretry(fake_args), 0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Global configuration options expressed in environment variables."""
from os import getenv
from os.path import expanduser, join

CONFIG_FILE = getenv(
    'NOTIFICATIONS_CONFIG',
    join(
        expanduser('~'),
        '.pacifica-notifications',
        'config.ini'
    )
)
CHERRYPY_CONFIG = getenv(
    'NOTIFICATIONS_CPCONFIG',
    join(
        expanduser('~'),
        '.pacifica-notifications',
        'cpconfig.ini'
    )
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Configuration reading and validation module."""
from os import getenv
try:
    from ConfigParser import SafeConfigParser
except ImportError:  # pragma: no cover python 2 vs 3 issue
    from configparser import ConfigParser as SafeConfigParser
from pacifica.notifications.globals import CONFIG_FILE


def get_config():
    """Return the ConfigParser object with defaults set."""
    configparser = SafeConfigParser({
        'default_user': getenv(
            'DEFAULT_USER',
            'default_user'
        )
    })
    configparser.add_section('celery')
    configparser.set('celery', 'broker_url', getenv(
        'BROKER_URL', 'redis://localhost:6379/0'))
    configparser.set('celery', 'backend_url', getenv(
        'BACKEND_URL', 'rpc://'))
    configparser.add_section('notifications')
    configparser.set('notifications', 'user_header', getenv(
        'USER_HEADER', 'Http-Remote-User'))
    configparser.set('notifications', 'policy_url', getenv(
        'POLICY_URL', 'http://localhost:8181'))
    configparser.add_section('database')
    configparser.set('database', 'peewee_url', getenv(
        'PEEWEE_URL', 'postgres://notifications:notifications@localhost/pacifica_notifications'))
    configparser.set('database', 'connect_attempts', getenv(
        'DATABASE_CONNECT_ATTEMPTS', '10'))
    configparser.set('database', 'connect_wait', getenv(
        'DATABASE_CONNECT_WAIT', '20'))
    configparser.read(CONFIG_FILE)
    return configparser



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The main module for executing the CherryPy server."""
import os
from sys import argv as sys_argv
from datetime import datetime, timedelta
from time import sleep
from argparse import ArgumentParser, SUPPRESS
from threading import Thread
import cherrypy
from peewee import OperationalError
from .tasks import dispatch_orm_event
from .orm import OrmSync, EventLog, NotificationSystem, eventget, eventpurge, SCHEMA_MAJOR, SCHEMA_MINOR
from .rest import Root, error_page_default
from .globals import CHERRYPY_CONFIG, CONFIG_FILE


def stop_later(doit=False):
    """Used for unit testing stop after 60 seconds."""
    if not doit:  # pragma: no cover
        return

    def sleep_then_exit():
        """
        Sleep for 20 seconds then call cherrypy exit.

        Hopefully this is long enough for the end-to-end tests to finish
        """
        sleep(20)
        cherrypy.engine.exit()
    sleep_thread = Thread(target=sleep_then_exit)
    sleep_thread.daemon = True
    sleep_thread.start()


def bool2cmdint(command_bool):
    """Convert a boolean to either 0 for true  or -1 for false."""
    if command_bool:
        return 0
    return -1


def _eventget(args):
    """Parse the dates and validate types before calling real eventget."""
    if not args.start:
        args.start = (datetime.now() - timedelta(days=1))
    else:
        args.start = datetime.strptime(args.start, args.format)
    if not args.end:
        args.end = datetime.now()
    else:
        args.end = datetime.strptime(args.end, args.format)
    if args.start > args.end:
        raise ValueError('Start time {} should be less than end time {}'.format(args.start, args.end))
    return bool2cmdint(eventget(args))


def _eventpurge(args):
    """Parse the dates and validate types before calling real eventpurge."""
    if not args.older_than:
        args.older_than = (datetime.now() - timedelta(days=1))
    else:
        args.older_than = datetime.strptime(args.older_than, args.format)
    return bool2cmdint(eventpurge(args))


def _eventretry(args):
    """Retry events given on the cmdline."""
    EventLog.database_connect()
    event_objs = list(EventLog.select().where(EventLog.uuid << args.events).execute())
    EventLog.database_close()
    results = []
    for event_obj in event_objs:
        results.extend(dispatch_orm_event(event_obj))
    sleep(3)
    for result in results:
        print('{} - {}'.format(result.task_id, result.wait(disable_sync_subtasks=False)))
    return 0


def _add_dbsync_parser(subparsers):
    """Add the dbsync subcommand arguments."""
    db_parser = subparsers.add_parser(
        'dbsync',
        description='Update or Create the Database.'
    )
    db_parser.set_defaults(func=dbsync)


def _add_dbchk_parser(subparsers):
    """Add the dbchk subcommand arguments."""
    dbchk_parser = subparsers.add_parser(
        'dbchk',
        description='Check database against current version.'
    )
    dbchk_parser.add_argument(
        '--equal', default=False,
        dest='check_equal', action='store_true'
    )
    dbchk_parser.set_defaults(func=dbchk)


def _add_eventget_parser(subparsers):
    """Add the eventget subcommand arguments."""
    dbchk_parser = subparsers.add_parser(
        'eventget',
        description='Get events from the log.'
    )
    dbchk_parser.add_argument(
        'events', nargs='*', type=str, help='Events to get'
    )
    dbchk_parser.add_argument(
        '--limit', default=10,
        dest='limit', type=int, help='Number of events to get'
    )
    dbchk_parser.add_argument(
        '--date-format', default='%Y-%m-%d %H:%M:%S',
        dest='format', type=str, help='Date format for start/end times'
    )
    dbchk_parser.add_argument(
        '--start-date', default=None,
        dest='start', type=str, help='Start date and time'
    )
    dbchk_parser.add_argument(
        '--end-date', default=None,
        dest='end', type=str, help='End date and time'
    )
    dbchk_parser.set_defaults(func=_eventget)


def _add_eventpurge_parser(subparsers):
    """Add the eventpurge subcommand arguments."""
    dbchk_parser = subparsers.add_parser(
        'eventpurge',
        description='Purge events from the log.'
    )
    dbchk_parser.add_argument(
        'events', nargs='*', type=str, help='Events to purge'
    )
    dbchk_parser.add_argument(
        '--date-format', default='%Y-%m-%d %H:%M:%S',
        dest='format', type=str, help='Date format for start/end times'
    )
    dbchk_parser.add_argument(
        '--older-than-date', default=None,
        dest='older_than', type=str, help='Events older than given date'
    )
    dbchk_parser.set_defaults(func=_eventpurge)


def _add_eventretry_parser(subparsers):
    """Add the eventretry subcommand arguments."""
    dbchk_parser = subparsers.add_parser(
        'eventretry',
        description='Retry events from the log.'
    )
    dbchk_parser.add_argument(
        'events', nargs='*', type=str, help='Events to purge'
    )
    dbchk_parser.set_defaults(func=_eventretry)


def cmd(*argv):
    """Admin command line tool."""
    parser = ArgumentParser(description='Notifications admin tool.')
    parser.add_argument(
        '-c', '--config', metavar='CONFIG', type=str, default=CONFIG_FILE,
        dest='config', help='notifications config file'
    )
    parser.set_defaults(func=lambda _args: parser.print_help())
    subparsers = parser.add_subparsers(help='sub-command help')
    _add_dbsync_parser(subparsers)
    _add_dbchk_parser(subparsers)
    _add_eventget_parser(subparsers)
    _add_eventpurge_parser(subparsers)
    _add_eventretry_parser(subparsers)
    if not argv:  # pragma: no cover
        argv = sys_argv[1:]
    args = parser.parse_args(argv)
    return args.func(args)


def main(*argv):
    """Main method to start the httpd server."""
    parser = ArgumentParser(description='Run the notifications server.')
    parser.add_argument(
        '-c', '--config', metavar='CONFIG', type=str,
        default=CONFIG_FILE, dest='config',
        help='notifications config file'
    )
    parser.add_argument(
        '--cpconfig', metavar='CONFIG', type=str,
        default=CHERRYPY_CONFIG, dest='cpconfig',
        help='cherrypy config file'
    )
    parser.add_argument(
        '-p', '--port', metavar='PORT', type=int, default=8070, dest='port',
        help='port to listen on'
    )
    parser.add_argument(
        '-a', '--address', metavar='ADDRESS', default='localhost',
        dest='address', help='address to listen on'
    )
    parser.add_argument(
        '--stop-after-a-moment', help=SUPPRESS, default=False,
        dest='stop_later', action='store_true'
    )
    if not argv:  # pragma: no cover
        argv = sys_argv[1:]
    args = parser.parse_args(argv)
    OrmSync.dbconn_blocking()
    if not NotificationSystem.is_safe():
        raise OperationalError('Database version too old {} update to {}'.format(
            '{}.{}'.format(*(NotificationSystem.get_version())),
            '{}.{}'.format(SCHEMA_MAJOR, SCHEMA_MINOR)
        ))
    stop_later(args.stop_later)
    cherrypy.config.update({'error_page.default': error_page_default})
    cherrypy.config.update({
        'server.socket_host': args.address,
        'server.socket_port': args.port
    })
    cherrypy.quickstart(Root(), '/', args.cpconfig)


def dbsync(args):
    """Create/Update the database schema to current code."""
    os.environ['NOTIFICATIONS_CONFIG'] = args.config
    OrmSync.dbconn_blocking()
    return bool2cmdint(OrmSync.update_tables())


def dbchk(args):
    """Check to see if the database is safe to use."""
    os.environ['NOTIFICATIONS_CONFIG'] = args.config
    OrmSync.dbconn_blocking()
    if args.check_equal:
        return bool2cmdint(NotificationSystem.is_equal())
    return bool2cmdint(NotificationSystem.is_safe())



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the example module."""
from json import dumps
from time import sleep
import threading
import requests
import cherrypy
from celery.bin.celery import main as celery_main
from pacifica.notifications.orm import EventMatch, EventLogMatch, EventLog, NotificationSystem, DB
from pacifica.notifications.rest import Root, error_page_default
from pacifica.notifications.globals import CHERRYPY_CONFIG


def eventmatch_droptables(func):
    """Setup the database and drop it once done."""
    def wrapper(*args, **kwargs):
        """Create the database table."""
        for orm_obj in [EventLogMatch, EventMatch, EventLog]:
            orm_obj.database_connect()
            if orm_obj.table_exists():
                orm_obj.drop_table()
        for orm_obj in [EventLog, EventMatch, EventLogMatch]:
            orm_obj.create_table()
        func(*args, **kwargs)
        for orm_obj in [EventLogMatch, EventMatch, EventLog]:
            orm_obj.drop_table()
    return wrapper


class NotificationsCPTest:
    """Base class for all testing classes."""

    HOST = '127.0.0.1'
    PORT = 8070
    url = 'http://{0}:{1}'.format(HOST, PORT)
    headers = {'content-type': 'application/json'}

    @staticmethod
    def setup_server():
        """Bind tables to in memory db and start service."""
        cherrypy.config.update({'error_page.default': error_page_default})
        cherrypy.config.update(CHERRYPY_CONFIG)
        cherrypy.tree.mount(Root(), '/', CHERRYPY_CONFIG)

    def _create_eventmatch(self, **kwargs):
        """Create a test eventmatch and return resp."""
        local_headers = kwargs.get('headers', {})
        local_headers.update(self.headers)
        local_data = {
            'name': 'testevent',
            'jsonpath': '$.data.*[?(@.value="Blah")].value',
            'target_url': 'http://127.0.0.1:8080',
            'auth': {
                'type': 'basic',
                'basic': {
                    'username': 'dmlb2001',
                    'password': 'password'
                }
            }
        }
        local_data.update(kwargs.get('json_data', {}))
        return requests.post(
            '{}/eventmatch'.format(self.url),
            data=dumps(local_data),
            headers=local_headers
        )

    # pylint: disable=invalid-name
    def setUp(self):
        """Setup the database with in memory sqlite."""
        DB.drop_tables([EventLogMatch, EventLog, EventMatch, NotificationSystem], safe=True)

        def run_celery_worker():
            """Run the main solo worker."""
            return celery_main([
                'celery', '-A', 'pacifica.notifications.tasks', 'worker', '--pool', 'solo',
                '--quiet', '-b', 'redis://127.0.0.1:6379/0'
            ])

        self.celery_thread = threading.Thread(target=run_celery_worker)
        self.celery_thread.start()
        sleep(3)

    # pylint: disable=invalid-name
    def tearDown(self):
        """Tear down the test and remove local state."""
        try:
            celery_main([
                'celery', '-A', 'pacifica.notifications.tasks', 'control',
                '-b', 'redis://127.0.0.1:6379/0', 'shutdown'
            ])
        except SystemExit:
            pass
        self.celery_thread.join()
        try:
            celery_main([
                'celery', '-A', 'pacifica.notifications.tasks', '-b', 'redis://127.0.0.1:6379/0',
                '--force', 'purge'
            ])
        except SystemExit:
            pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the example module."""
from json import dumps
import requests
from cherrypy.test import helper
from .common_test import NotificationsCPTest, eventmatch_droptables


class EventMatchCPTest(NotificationsCPTest, helper.CPWebCase):
    """Test the EventMatch class."""

    @eventmatch_droptables
    def test_create(self):
        """Test the create POST method in EventMatch."""
        resp = self._create_eventmatch()
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertTrue('uuid' in resp.json())

    @eventmatch_droptables
    def test_delete(self):
        """Test the delete method in EventMatch."""
        resp = self._create_eventmatch()
        uuid = resp.json()['uuid']
        resp = requests.delete(
            '{}/eventmatch/{}'.format(self.url, uuid)
        )
        self.assertEqual(resp.status_code, 200)

    @eventmatch_droptables
    def test_get_single(self):
        """Test the delete method in EventMatch."""
        resp = self._create_eventmatch()
        uuid = resp.json()['uuid']
        resp = requests.get(
            '{}/eventmatch/{}'.format(self.url, uuid)
        )
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()['uuid'], uuid)

    @eventmatch_droptables
    def test_get_list(self):
        """Test the delete method in EventMatch."""
        resp = self._create_eventmatch()
        uuid = resp.json()['uuid']
        resp = requests.get(
            '{}/eventmatch'.format(self.url)
        )
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()[0]['uuid'], uuid)

    @eventmatch_droptables
    def test_update(self):
        """Test the delete method in EventMatch."""
        resp = self._create_eventmatch()
        uuid = resp.json()['uuid']
        resp = requests.put(
            '{}/eventmatch/{}'.format(self.url, uuid),
            data=dumps({
                'target_url': 'http://example.com/1234'
            }),
            headers=self.headers
        )
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()['uuid'], uuid)
        self.assertEqual(resp.json()['target_url'], 'http://example.com/1234')

    @eventmatch_droptables
    def test_not_found(self):
        """Test the delete method in EventMatch."""
        for uuid in ['', '6cf47199-5abf-4ae3-a40d-9d6283d0ee21']:
            resp = requests.get('{}/eventmatch/{}'.format(self.url, uuid))
            self.assertEqual(resp.status_code, 403)
            self.assertTrue('Content-Type' in resp.headers)
            self.assertEqual(resp.headers['Content-Type'], 'application/json')
            self.assertEqual(resp.json()['status'], '403 Forbidden')

    @eventmatch_droptables
    def test_bad_uuid(self):
        """Test the get with bad UUID."""
        resp = requests.get(
            '{}/eventmatch/1234'.format(self.url)
        )
        self.assertEqual(resp.status_code, 500)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()['status'], '500 Internal Server Error')

    @eventmatch_droptables
    def test_get_wrong_user(self):
        """Test the create POST method in EventMatch."""
        resp = self._create_eventmatch()
        resp = requests.get(
            '{}/eventmatch/{}'.format(self.url, resp.json()['uuid']),
            headers={'Http-Remote-User': 'dmlb2001'}
        )
        self.assertEqual(resp.status_code, 403)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()['status'], '403 Forbidden')

    @eventmatch_droptables
    def test_setting_alternate_user(self):
        """Test the create POST method in EventMatch."""
        resp = self._create_eventmatch(
            headers={'Http-Remote-User': 'dmlb2001'})
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()['user'], 'dmlb2001')

    @eventmatch_droptables
    def test_update_wrong_user(self):
        """Test the create POST method in EventMatch."""
        resp = self._create_eventmatch()
        resp = requests.put(
            '{}/eventmatch/{}'.format(self.url, resp.json()['uuid']),
            data=dumps({
                'target_url': 'http://foo.example.com/1234'
            }),
            headers={
                'Http-Remote-User': 'dmlb2001',
                'Content-Type': 'application/json'
            }
        )
        self.assertEqual(resp.status_code, 403)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()['status'], '403 Forbidden')

    @eventmatch_droptables
    def test_delete_wrong_user(self):
        """Test the delete method in EventMatch."""
        resp = self._create_eventmatch()
        uuid = resp.json()['uuid']
        resp = requests.delete(
            '{}/eventmatch/{}'.format(self.url, uuid),
            headers={
                'Http-Remote-User': 'dmlb2001',
                'Content-Type': 'application/json'
            }
        )
        self.assertEqual(resp.status_code, 403)
        self.assertTrue('Content-Type' in resp.headers)
        self.assertEqual(resp.headers['Content-Type'], 'application/json')
        self.assertEqual(resp.json()['status'], '403 Forbidden')



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the database connection logic."""
import os
from unittest import TestCase
import mock
import peewee
from pacifica.notifications.orm import EventMatch, OrmSync, NotificationSystem
from .common_test import NotificationsCPTest


class TestDBConnections(NotificationsCPTest, TestCase):
    """Testing the database connection utilities."""

    @mock.patch.object(EventMatch, 'database_connect')
    def test_bad_db_connection(self, mock_db_connect):
        """Test a failed db connection."""
        mock_db_connect.side_effect = peewee.OperationalError(
            mock.Mock(), 'Error')
        hit_exception = False
        os.environ['DATABASE_CONNECT_ATTEMPTS'] = '1'
        os.environ['DATABASE_CONNECT_WAIT'] = '1'
        try:
            OrmSync.dbconn_blocking()
        except peewee.OperationalError:
            hit_exception = True
        self.assertTrue(hit_exception)

    def test_no_table_goc_version(self):
        """Test the get or create version with no table."""
        OrmSync.dbconn_blocking()
        OrmSync.update_tables()
        NotificationSystem.drop_table()
        major, minor = NotificationSystem.get_or_create_version()
        self.assertEqual(major, 0)
        self.assertEqual(minor, 0)
